@@ -1,7 +1,12 @@
+"""
+This module handles server and client certificates for ajenti.
+"""
+
 import json
 import random
 import socket
-from OpenSSL.crypto import *
+import OpenSSL.crypto
+import base64
 
 import aj
 from jadi import component
@@ -18,13 +23,23 @@ class Handler(HttpPlugin):
     @url(r'/api/settings/generate-client-certificate')
     @endpoint(api=True)
     def handle_api_generate_client_certificate(self, http_context):
+        """
+        Generate new client certificate based on settings informations.
+        Method POST.
+
+        :param http_context: HttpContext
+        :type http_context: HttpContext
+        :return: Certificates informations (serial, digest, ...)
+        :rtype: dict
+        """
+
         data = json.loads(http_context.body.decode())
 
-        key = PKey()
-        key.generate_key(TYPE_RSA, 4096)
-        ca_key = load_privatekey(FILETYPE_PEM, open(aj.config.data['ssl']['certificate']).read())
-        ca_cert = load_certificate(FILETYPE_PEM, open(aj.config.data['ssl']['certificate']).read())
-        cert = X509()
+        key = OpenSSL.crypto.PKey()
+        key.generate_key(OpenSSL.crypto.TYPE_RSA, 4096)
+        ca_key = OpenSSL.crypto.load_privatekey(OpenSSL.crypto.FILETYPE_PEM, open(aj.config.data['ssl']['certificate']).read())
+        ca_cert = OpenSSL.crypto.load_certificate(OpenSSL.crypto.FILETYPE_PEM, open(aj.config.data['ssl']['certificate']).read())
+        cert = OpenSSL.crypto.X509()
         cert.get_subject().countryName = data['c']
         cert.get_subject().stateOrProvinceName = data['st']
         cert.get_subject().organizationName = data['o']
@@ -36,27 +51,38 @@ class Handler(HttpPlugin):
         cert.set_issuer(ca_cert.get_subject())
         cert.sign(ca_key, 'sha1')
 
-        pkcs = PKCS12()
+        pkcs = OpenSSL.crypto.PKCS12()
         pkcs.set_certificate(cert)
         pkcs.set_privatekey(key)
-        pkcs.set_friendlyname(str(data['cn']))
+        pkcs.set_friendlyname(bytes(data['cn'], encoding="utf-8"))
 
         return {
-            'digest': cert.digest('sha1'),
-            'name': ','.join('%s=%s' % x for x in cert.get_subject().get_components()),
+            'digest': cert.digest('sha1').decode('utf-8'),
+            'name': ','.join(b'='.join(x).decode('utf-8')
+                             for x in cert.get_subject().get_components()
+                             ),
             'serial': str(cert.get_serial_number()),
-            'b64certificate': pkcs.export().encode('base64'),
+            'b64certificate': base64.b64encode(pkcs.export()).decode('utf-8'),
         }
 
     @url(r'/api/settings/generate-server-certificate')
     @endpoint(api=True)
     def handle_api_generate_server_certificate(self, http_context):
+        """
+        Generate a new server certificate for a SSL connection to ajenti server.
+
+        :param http_context: HttpContext
+        :type http_context: HttpContext
+        :return: Certificate path commonly /etc/ajenti/ajenti.pem
+        :rtype: dict
+        """
+
         etc_dir = '/etc/ajenti'
         certificate_path = '%s/ajenti.pem' % etc_dir
 
-        key = PKey()
-        key.generate_key(TYPE_RSA, 4096)
-        cert = X509()
+        key = OpenSSL.crypto.PKey()
+        key.generate_key(OpenSSL.crypto.TYPE_RSA, 4096)
+        cert = OpenSSL.crypto.X509()
         cert.get_subject().countryName = 'NA'
         cert.get_subject().organizationName = socket.gethostname()
         cert.get_subject().commonName = 'ajenti'
@@ -68,8 +94,8 @@ class Handler(HttpPlugin):
         cert.sign(key, 'sha1')
 
         with open(certificate_path, 'wb') as f:
-            f.write(dump_privatekey(FILETYPE_PEM, key))
-            f.write(dump_certificate(FILETYPE_PEM, cert))
+            f.write(OpenSSL.crypto.dump_privatekey(OpenSSL.crypto.FILETYPE_PEM, key))
+            f.write(OpenSSL.crypto.dump_certificate(OpenSSL.crypto.FILETYPE_PEM, cert))
 
         return {
             'path': certificate_path,
@@ -78,12 +104,22 @@ class Handler(HttpPlugin):
     @url(r'/api/settings/test-certificate/')
     @endpoint(api=True)
     def handle_test_certificate(self, http_context):
+        """
+        Try to load the specified certificate in order to test if it is valid
+        or not.
+
+        :param http_context: HttpContext
+        :type http_context: HttpContext
+        :return: Certificate path, commonly in /etc/ajenti
+        :rtype: dict
+        """
+
         if http_context.method == 'POST':
             data = http_context.json_body()
             certificate_path = data['certificate']
             try:
-                certificate = load_certificate(FILETYPE_PEM, open(certificate_path).read())
-                private_key = load_privatekey(FILETYPE_PEM, open(certificate_path).read())
+                OpenSSL.crypto.load_certificate(OpenSSL.crypto.FILETYPE_PEM, open(certificate_path).read())
+                OpenSSL.crypto.load_privatekey(OpenSSL.crypto.FILETYPE_PEM, open(certificate_path).read())
             except Exception as e:
                 raise EndpointError(None, message=str(e))    
                 
